
'''

wlnmd.py
==========

Desc: run wlnm deamon as SimpleXMLRPCServer


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


from SimpleXMLRPCServer import SimpleXMLRPCServer
from daemon import Daemon
from os.path import expanduser
import os


import remoter
import util

def list_contents(dir_name):
	return os.listdir(dir_name)

def searchAll():
	return remoter.searchAll()

def show(args):
	return remoter.show(args)

def showWLS():
	return remoter.showWLS()

def showDomains():
	return remoter.showDomains()

def nmstart(args):
	return remoter.nmstart(args)

def startAdmin(args):
	return remoter.startAdmin(args)

def startServers(args):
	return remoter.startServers(args)

def stopServers(args):
	return remoter.stopServers(args)

def listprocess():
	return remoter.listprocess()

def kill(args):
	return remoter.kill(args)

wlnmd_pid_file = "%s/.wlnm/wlnm.pid" % expanduser("~")

class wlnmDaemon(Daemon):
	
	
	def __init__(self,pidfile, port,stdin=os.devnull, stdout=os.devnull, stderr=os.devnull, home_dir='.', umask=022, verbose=1):
		wlnmd_out_file = "%s/.wlnm/wlnm.out" % expanduser("~")
		self.stdin = stdin
		self.stdout = wlnmd_out_file
		self.stderr = wlnmd_out_file
		self.pidfile = pidfile
		self.home_dir = home_dir
		self.verbose = verbose
		self.umask = umask
		self.daemon_alive = True
		self.port = port


        def run(self):
                server = SimpleXMLRPCServer(('0.0.0.0', self.port), logRequests=True)
		server.register_function(list_contents)
		server.register_function(searchAll)
		server.register_function(show)
		server.register_function(showWLS)
		server.register_function(showDomains)
		server.register_function(nmstart)
		server.register_function(startAdmin)
		server.register_function(startServers)
		server.register_function(stopServers)
		server.register_function(listprocess)
		server.register_function(kill)

		try:
			#print 'Use Control-C to exit'
			server.serve_forever()
		except KeyboardInterrupt:
			print 'Exiting'

	



def runDaemon(iport):
	
	daemon = wlnmDaemon(pidfile=wlnmd_pid_file,port=iport)
	daemon.start()
	print "wlnm daemon "


def stopDaemon():
	
	if os.path.isfile(wlnmd_pid_file):
	    with open(wlnmd_pid_file) as f:
		pids = f.readlines()
	    try:
		os.kill(int(pids[0]),9)	#force kill
		print "wlnmd process %s is stopped ." % int(pids[0])
	    except :
		print "wlnmd process %s was killed before ." % int(pids[0])
	    finally :
		os.remove(wlnmd_pid_file)
	else :
	    print "wlnmd is not running , no process can be stopped."
	
	print

#runDaemon(9099)
#stopDaemon()



'''

inputer.py
==========

Desc: Helper module for command which user typed


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import readline

import search
import starter
import monitor
import wls
import util
import wlnmc
from wlnmd import wlnmDaemon


remote = False

cmddict={'init':search.searchAll,
	'ls':search.show,
	'lswls':search.showWLS,
	'lsd':search.showDomains,
	'nmstart':starter.nmstart,
	#'checkport':util.checkport,
	'help':util.help,
	'startadmin':starter.startAdmin,
	'start':wls.startServers,
	'stop':wls.stopServers,
	'lsp':monitor.listprocess,
	'kill':starter.kill,
	'connect':wlnmc.connect,
	'quit':sys.exit
}

remote_cmddict={'init':wlnmc.searchAll,
	'ls':wlnmc.show,
	'lswls':wlnmc.showWLS,
	'lsd':wlnmc.showDomains,
	'nmstart':wlnmc.nmstart,
	'help':util.help,
	'startadmin':wlnmc.startAdmin,
	'start':wlnmc.startServers,
	'stop':wlnmc.stopServers,
	'lsp':wlnmc.listprocess,
	'kill':wlnmc.kill,
	'connect':wlnmc.connect,
	'disconnect':wlnmc.disconnect,
	'quit':sys.exit
}

cmds=cmddict.keys()

rcmds = remote_cmddict.keys()

def complete(text, state):
    if remote :
	for cmd in rcmds:
		if cmd.startswith(text):
		    if not state:
			return cmd
		    else:
			state -= 1
    else:
	for cmd in cmds:
		if cmd.startswith(text):
		    if not state:
			return cmd
		    else:
			state -= 1

def validcmd(command,args,remote):
	if command in ["lsd" ,"lsp" ,"lswls" , "init" , "quit","disconnect","help"] and args :
		print "no arguments for command %s" % command
		return
	if command in ["nmstart" ,"startadmin" ,"start" , "stop" , "kill","connect"] and not args :
		print "arguments needed run command %s" % command
		return
	runcmd(command,args,remote)

def runcmd(command,args,remote):
	
	if remote :
		icmddict = remote_cmddict
	else :
		icmddict = cmddict

	if command not in icmddict.keys():
		print 'Invilad command! Press Tab key or use help command list all avaiable command.'
		print ''
	else:
		if not args :
			if command == "ls" :
				icmddict[command]([])
			else :
				icmddict[command]()
		else:
			icmddict[command](args)




'''

stopwls.py
==========

Desc: WLST script for stoping remote/local managed server 


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import os
from java.lang import System
import getopt

admin_username = '';
admin_password = '';
admin_server_url ='';
server_name = '';

def usage():
    print "Usage:"
    print "wlst.sh stopwls.py -u <user> -p <pass> -a <adminurl> -s <servername>"

try:
    opts, args = getopt.getopt( sys.argv[1:], "a:p:u:s:", "" )
   
    
except getopt.GetoptError, err:
    print str(err)
    usage()
    sys.exit(2)

for opt, arg in opts:
	if opt == "-u":
		admin_username = arg
	elif opt == "-p":
		admin_password = arg
	elif opt == "-a":
		admin_server_url = arg
	elif opt == "-s":
		server_name = arg

#print '---->'+admin_username+' '+admin_password+' '+admin_server_url+' '+server_name;


print 'CONNECT TO ADMIN URL '+admin_server_url+'.......';
try:
	connect(admin_username, admin_password, admin_server_url);
except:
	print 'CAN NOT CONNECT ADMIN SERVER';


print 'STOPPING WEBLOGIC SERVER  '+server_name+'.......';
try:
	shutdown(server_name,'Server','false',1000,'true');
except:
	print 'FAIL TO STOP SERVER , IT MAY ALREADY SHUTDOWN';





'''

wls.py
==========

Desc: WLST proxy module to invoke WLST script 


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import subprocess
import logging
import os
import search

logging.basicConfig(level=logging.INFO)
#logging.basicConfig(level=logging.DEBUG)

WLSTPATH = os.path.dirname(search.__file__)

def startServers(namelist):
	logging.debug ("namelist is %s" % namelist)
	for name in namelist:
		startServer(name)


def startServer(name):
	logging.debug ("name is %s" % name)
	flag = False
	(adminurl,wlsHome,user,pwd) = search.getwls(name)

	if wlsHome :
		cmd_str = '%s/common/bin/wlst.sh %s/wlst/startwls.py -u %s -p %s -a %s -s %s' % (wlsHome,WLSTPATH,user,pwd,adminurl,name)
		proc = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
		
		print "%s starting request has been sent , will be in running status shortly. " % name
		'''
		# not print output in this way ,as start may takes time
		while proc.poll() is None:
		    output = proc.stdout.readline()
		    if output.startswith('CONNECT TO ADMIN URL') or flag:
			flag = True
		    if flag:
			print output
		'''
	else:
		print "could not find %s  , make sure you spell correct" % name
	

def stopServers(namelist):
	logging.debug ("namelist is %s" % namelist)
	for name in namelist:
		stopServer(name)

def stopServer(name):
	logging.debug ("name is %s" % name)
	(adminurl,wlsHome,user,pwd) = search.getwls(name)
	flag = False
	if wlsHome :
		cmd_str = '%s/common/bin/wlst.sh %s/wlst/stopwls.py -u %s -p %s -a %s -s %s' % (wlsHome,WLSTPATH,user,pwd,adminurl,name)
		proc = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
		while proc.poll() is None:
		    output = proc.stdout.readline()
		    if output.startswith('CONNECT TO ADMIN URL') or flag:
			flag = True
		    if flag:
			print output
	else:
		print "could not find %s , make sure you spell correct" % name




def getAdminPass(domainhome,encryptpass,wlsHome):
	
	result = ''
	cmd_str = '%s/common/bin/wlst.sh %s/wlst/decryptPass.py %s %s' % (wlsHome,WLSTPATH,domainhome,encryptpass)
	proc = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
	while proc.poll() is None:
	    _tmp = proc.stdout.readline()
	    result = _tmp if _tmp else result
	    #print _tmp
	return result.strip()

	
#print getAdminPass('/beavar/1035/domains/wlsd_poc_local','{AES}wFHEVrsOI5zYvqadICOAqkOZghorXgTMJ79jiGufnIY=','/bea/1035/wlserver_10.3')


#!/usr/bin/env python

'''

wlnm.py
==========

Desc: Entry point of weblogic node master 


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import sys
import os
#import re
#import subprocess,shlex
import logging
import argparse
import socket
import readline

import inputer,search,wlnmd,wlnmc



logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s')


def local_run():

    print "Weblogic Node Master"
    print ""
    print "Type help to load help page ."
    print ""

    command =''
    #remoteCMD = False
    readline.parse_and_bind("tab: complete")
    while True :
        #remoteCMD = inputer.remote
	readline.set_completer(inputer.complete)
	if inputer.remote :
		i_input = raw_input ('%s>>' % wlnmc.host)
	else:
		i_input = raw_input ('%s(localhost)>>' % socket.gethostname())
	if not i_input : continue
	command = i_input.split()[0]
	args=i_input.split()[1:]
	inputer.validcmd(command,args,inputer.remote)
	
	'''
	if command == "connect" :
		remoteCMD = True
	if command == "disconnect" :
		remoteCMD = False
	'''
    print "Bye"



def main():
    print 
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--daemon', action="store_true" , dest='daemon', default=False, help='Run wlnm in daemon mode')
    parser.add_argument('-p', '--port', dest='listenPort', help='Listen port to run wlnm as daemon')
    parser.add_argument('-k', '--kill', action="store_true", dest='killwlnmd', help='Kill wlnm daemon process')
    args = parser.parse_args()
    
    if args.daemon or args.killwlnmd :
	search.checkinit(True)
    else :
	search.checkinit(False)
    
    if  args.daemon:
	if args.listenPort :
		#print args.listenPort 
		
		wlnmd.runDaemon(int(args.listenPort))
	else:
		print "Port number need to provide , eg. -p 9099 or --port 9099"
	
    elif  args.killwlnmd:
	wlnmd.stopDaemon()
    else :
	local_run()


if __name__ == "__main__":
    main()



'''

code from http://www.jejik.com/articles/2007/02/a_simple_unix_linux_daemon_in_python/

'''

import sys, os, time, atexit
from signal import SIGTERM
 
class Daemon:
        """
        A generic daemon class.
       
        Usage: subclass the Daemon class and override the run() method
        """
        def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
                self.stdin = stdin
                self.stdout = stdout
                self.stderr = stderr
                self.pidfile = pidfile
       
        def daemonize(self):
                """
                do the UNIX double-fork magic, see Stevens' "Advanced
                Programming in the UNIX Environment" for details (ISBN 0201563177)
                http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
                """
                try:
                        pid = os.fork()
                        if pid > 0:
                                # exit first parent
                                sys.exit(0)
                except OSError, e:
                        sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
                        sys.exit(1)
       
                # decouple from parent environment
                os.chdir("/")
                os.setsid()
                os.umask(0)
       
                # do second fork
                try:
                        pid = os.fork()
                        if pid > 0:
                                # exit from second parent
                                sys.exit(0)
                except OSError, e:
                        sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
                        sys.exit(1)
       
                # redirect standard file descriptors
                sys.stdout.flush()
                sys.stderr.flush()
                si = file(self.stdin, 'r')
                so = file(self.stdout, 'a+')
                se = file(self.stderr, 'a+', 0)
                os.dup2(si.fileno(), sys.stdin.fileno())
                os.dup2(so.fileno(), sys.stdout.fileno())
                os.dup2(se.fileno(), sys.stderr.fileno())
       
                # write pidfile
                atexit.register(self.delpid)
                pid = str(os.getpid())
                file(self.pidfile,'w+').write("%s\n" % pid)
       
        def delpid(self):
                os.remove(self.pidfile)
 
        def start(self):
                """
                Start the daemon
                """
                # Check for a pidfile to see if the daemon already runs
                try:
                        pf = file(self.pidfile,'r')
                        pid = int(pf.read().strip())
                        pf.close()
                except IOError:
                        pid = None
       
                if pid:
                        message = "pidfile %s already exist. Daemon already running?\n"
                        sys.stderr.write(message % self.pidfile)
                        sys.exit(1)
               
                # Start the daemon
                self.daemonize()
                self.run()
 
        def stop(self):
                """
                Stop the daemon
                """
                # Get the pid from the pidfile
                try:
                        pf = file(self.pidfile,'r')
                        pid = int(pf.read().strip())
                        pf.close()
                except IOError:
                        pid = None
       
                if not pid:
                        message = "pidfile %s does not exist. Daemon not running?\n"
                        sys.stderr.write(message % self.pidfile)
                        return # not an error in a restart
 
                # Try killing the daemon process       
                try:
                        while 1:
                                os.kill(pid, SIGTERM)
                                time.sleep(0.1)
                except OSError, err:
                        err = str(err)
                        if err.find("No such process") > 0:
                                if os.path.exists(self.pidfile):
                                        os.remove(self.pidfile)
                        else:
                                print str(err)
                                sys.exit(1)
 
        def restart(self):
                """
                Restart the daemon
                """
                self.stop()
                self.start()
 
        def run(self):
                """
                You should override this method when you subclass Daemon. It will be called after the process has been
                daemonized by start() or restart().
                """




'''

util.py
==========

Desc: Utils including data store , table printing ,formating and help page


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import socket
import shelve
from os.path import expanduser
from cStringIO import StringIO
import os
import sys
import base64


wlnm_data_file = "%s/.wlnm/wlnm.dat" % expanduser("~")

def creatProfileDir():
	pdir = "%s/.wlnm" % expanduser("~")
	if not os.path.exists(pdir):
		os.makedirs(pdir,0700)


def help():
	print ">>>>>> help >>>>>>"
	print ""
	print ""
	print "ls [domainName]		List all weblogic domains/servers or only list one domain. "
	print ""
	print "lsd			List all weblogic domains."
	print ""
	print "lsp			List all running weblogic instances."
	print ""
	print "lswls			List all version weblogic installed on local machine."
	print ""
	print "kill [port|servername]	Kill process by port number or servername."
	print ""
	print "init			Reinitialize weblogic and domain info on local machine."
	print ""
	print "nmstart [port]		start weblogic node manager by port."
	print ""
	print "startadmin [domainname]	start admin server by domain name."
	print ""
	print "start [servername]	start managed server by servername."
	print ""
	print "stop [servername]	stop managed server by servername."
	print ""
	print "help			Show this page."
	print ""
	print "quit			quit weblogic node master."
	print ""
	print ""
	


#locale.setlocale(locale.LC_NUMERIC, "")
def format_num(num):
    """Format a number according to given places.
    Adds commas, etc. Will truncate floats into ints!"""

    #try:
    #    inum = int(num)
    #    return locale.format("%.*f", (0, inum), True)

    #except (ValueError, TypeError):
    return str(num)

def get_max_width(table, index):
    """Get the maximum width of the given column index"""
    return max([len(format_num(row[index])) for row in table])


def pprint_table(table):
    """Prints out a table of data, padded for alignment
    @param out: Output stream (file-like object)
    @param table: The table to print. A list of lists.
    Each row must have the same number of columns. """
    print ''

    col_paddings = []

    for i in range(len(table[0])):
        col_paddings.append(get_max_width(table, i))

    rownum = 0 
    for row in table:
	rownum += 1 
	if rownum == 1 :
		print '\033[1m'
        # left col
        print "%s ||" % row[0].ljust(col_paddings[0] + 1),
        # rest of the cols
        for i in range(1, len(row)):
            col = format_num(row[i]).rjust(col_paddings[i] + 2)
	    print "%s |" %col,
	
	if rownum == 1 :
		print '\033[0m'
        else:
		print


    print

def checkports(port):
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	for iport in port:
		print iport
		if s.connect_ex(('localhost', int(iport)))==0:
			print "port %s is Opened" % iport
			#return True
		else:
			print "port %s is Closed" % iport
			#return False
		s.close()

def checkport(port):
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	if s.connect_ex(('localhost', int(port)))==0:
		#print "port %s is Opened" % iport
		return True
	else:
		#print "port %s is Closed" % iport
		return False
	s.close()

def initDB():
	db = shelve.open(wlnm_data_file,"c")
	db['local_wls'] = {}
	db['local_domains'] = {}
	db['wls_servers'] = {}
	db.close()

def loadDB(wlss=False,domain=False,server=False):
	
	db = shelve.open(wlnm_data_file, "r")
	wlsDB = None
	wls = None
	domainsDB = None
	domains = None
	serversDB = None
	servers = None

	if wlss :
		wlsDB = db['local_wls']
		wls = wlsDB.keys()
	if domain:
		domainsDB = db['local_domains']
		domains = domainsDB.keys()
	if server :
		serversDB = db['wls_servers']
		servers = serversDB.keys()
	db.close()
	return ([wlsDB,wls],[domainsDB,domains],[serversDB,servers])


def saveDB(wlss=False,domain=False,server=False,data=None):
	#print wlnm_data_file
	db = shelve.open(wlnm_data_file,"c")

	if wlss :
		wlsHome,version,nmPort = data[0]
		if 'local_wls' not in db.keys() :
			local_wls={}
		else:
			local_wls= db['local_wls']

		local_wls['wls'+version] = {"home":wlsHome,"version":version,"nmport":nmPort}
		db['local_wls']=local_wls
		#print "wls%s has been store in local db!" % version

	if domain :
		domainName,domainhome,version,adminurl,user,pwd = data[1]

		if 'local_domains' not in db.keys() :
			local_domains={}
		else:
			local_domains= db['local_domains']

		local_domains[domainName] = {"home":domainhome,"version":version,"adminurl":adminurl,"user":user,"pwd":pwd}
		db['local_domains']=local_domains
		#print "%s has been store in local db!" % domainName
	
	if server :
		name,host,port,type,domain = data[2]

		if 'wls_servers' not in db.keys() :
			wls_servers={}
		else:
			wls_servers= db['wls_servers']

		_key = "%s.%s" % (name,domain)
		wls_servers[_key] = {"name":name,"host":host,"port":port,"type":type,"domain":domain}
		db['wls_servers']=wls_servers
		#print "%s has been store in local db!" % _key

	

	db.close()



def redirectStdout():
	old_stdout = sys.stdout
	sys.stdout = mystdout = StringIO()
	return mystdout,old_stdout

def finishRedirectStdout(old_stdout):
	sys.stdout = old_stdout

def decode_output(my_out):
	return base64.b64decode(my_out)

def encode_output(my_out):
	return base64.b64encode(my_out)




'''

decryptPass.py
==========

Desc: code refer to Rafael Arana 's http://techtapas.blogspot.com.au/2011/05/how-to-decrypt-weblogic-passwords-with.html

Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import os
import weblogic.security.internal.SerializedSystemIni
import weblogic.security.internal.encryption.ClearOrEncryptedService

def usage():
    print "Usage:"
    print "wlst.sh decryptPass.py <domainhome> <encryptpass> "

def decrypt(domainhome, encryptpass):
    domainHomePath = os.path.abspath(domainhome)
    encryption = weblogic.security.internal.SerializedSystemIni.getEncryptionService(domainHomePath)
    wlces = weblogic.security.internal.encryption.ClearOrEncryptedService(encryption)
    plainpwd = wlces.decrypt(encryptpass)
    print plainpwd

try:
     decrypt(sys.argv[1], sys.argv[2])
except:
    print 'NA'




'''

starter.py
==========

Desc: AdminServer and node manager process admin command including start and kill


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


import subprocess
import psutil

import search
import util

def RepresentsInt(s):
    try: 
        int(s)
        return True
    except ValueError:
        return False

def startAdmin(domain):
	for domain_str in domain:
		_domain = search.getDomain(domain_str)

		if _domain is not None :
			domainhome = _domain['home']
			proc = subprocess.Popen('nohup %s/startWebLogic.sh 1>>/dev/null 2>&1 &' % domainhome, shell=True, stdout=subprocess.PIPE)
			#out, err = proc.communicate()
			#print out
			print "server process starting "
		else :
			print "%s not exist , please check name or reinit" % domain_str
			return

def kill(args):
	for arg in args:
		if RepresentsInt(arg):
			killbyport(arg)
		else:
			killbyname(arg)

def killbyname(name):
	for p in psutil.process_iter():
			for subcmd in p.cmdline :
				if '-Dweblogic.Name=' in subcmd and name == subcmd.split("=")[-1]:
					p.kill()
					print "%s is stopped!" % name

def killbyport(port):
	 pid = ''
	 if not isWlsPort(port):
		print "%s:Not a valid weblogic port !" % port
		return
	 cmd_str = "netstat -pan 2>/dev/null|grep %s|grep LISTEN|tr -s ' ' '/' |cut -f7 -d '/'|tail -1" % port
	 p = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
	 pid = p.communicate()[0].strip()

	 if RepresentsInt(pid):
		 proc = psutil.Process(int(pid))
		 proc.kill()
		 print "process listen on %s is killed ." % port
	 else:
		print "no process listen on %s!" % port

def nmstart(port):
	_port = port[0]
	wlsDB,wls = util.loadDB(wlss=True)[0]
	wls_home = ''
	for w in wls:
	    if _port == wlsDB[w]["nmport"]:
		wls_home = wlsDB[w]["home"]    

	if wls_home :
		cmd_str = 'nohup %s/server/bin/startNodeManager.sh 1>/dev/null 2>&1 &' % wls_home
		runcmd(cmd_str)
	else:
		print "%s is not valid nodemanager port !" % port

def runcmd(cmd_str):
	proc = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
	#while proc.poll() is None:
	    #output = proc.stdout.readline()
	    #print output
	print "Nodemanager started."

def isWlsPort(port):
	iswls = False
	[wlsDB,wls],[serversDB,servers] = util.loadDB(wlss=True,server=True)[0::2]
	#print [wlsDB,wls],[serversDB,servers] 
	wls_home = ''

	for w in wls:
	    if port == wlsDB[w]["nmport"]:
		iswls = True    
	

	for server in servers:
		if port == serversDB[server]["port"] :
			iswls = True
	
	return iswls


'''

remoter.py
==========

Desc: process function proxy for remote call


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 


'''

import util
import search
import starter
import monitor
import wls

def searchAll():
	mystdout,old_stdout = util.redirectStdout()
	search.searchAll()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def show(args):
	mystdout,old_stdout = util.redirectStdout()
	search.show(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def showWLS():
	mystdout,old_stdout = util.redirectStdout()
	search.showWLS()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def showDomains():
	mystdout,old_stdout = util.redirectStdout()
	search.showDomains()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def nmstart(args):
	mystdout,old_stdout = util.redirectStdout()
	starter.nmstart(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startAdmin(args):
	mystdout,old_stdout = util.redirectStdout()
	starter.startAdmin(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def startServers(args):
	mystdout,old_stdout = util.redirectStdout()
	wls.startServers(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def stopServers(args):
	mystdout,old_stdout = util.redirectStdout()
	wls.stopServers(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def listprocess():
	mystdout,old_stdout = util.redirectStdout()
	monitor.listprocess()
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())

def kill(args):
	mystdout,old_stdout = util.redirectStdout()
	starter.kill(args)
	util.finishRedirectStdout(old_stdout)
	return util.encode_output(mystdout.getvalue())






'''

wlnmc.py
==========

Desc:  wlnm client as SimpleXMLRPC client


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import xmlrpclib
import util
import inputer

def connect(args):
	global host
	global port
	global proxy

	
	try:
		host= args[0]
		port= args[1]
		proxy = xmlrpclib.ServerProxy('http://%s:%s' % (host,port))
		proxy.list_contents('.')
		inputer.remote = True
	except:
		print 'not able to connect to http://%s ,make sure port and host is corrected' % ':'.join(args)
		inputer.remote = False

def searchAll():
	print util.decode_output(proxy.searchAll())

def show(args):
	print util.decode_output(proxy.show(args))

def showWLS():
	print util.decode_output(proxy.showWLS())

def showDomains():
	print util.decode_output(proxy.showDomains())

def nmstart(args):
	print util.decode_output(proxy.nmstart(args))

def startAdmin(args):
	print util.decode_output(proxy.startAdmin(args))

def startServers(args):
	print util.decode_output(proxy.startServers(args))

def stopServers(args):
	print util.decode_output(proxy.stopServers(args))

def listprocess():
	print util.decode_output(proxy.listprocess())

def kill(args):
	print util.decode_output(proxy.kill(args))

def disconnect():
	proxy = None
	host = None
	port = None
	inputer.remote = False


#connect(["localhost","9099"])
#print proxy.list_contents('/tmp')


'''

search.py
==========

Desc: Functions for searching weblogic properties through config.xml and nodemanger.properties


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''


import os
import logging
import xml.etree.ElementTree as ET
import subprocess
import sys

import util
import monitor
import wls


logging.basicConfig(level=logging.INFO)
#logging.basicConfig(level=logging.DEBUG)

def get_xml_namespace(re):
	return ''.join(re.tag.split("}")[:-1])+'}'

def saveDomain(domainName,domainhome,version,adminurl,user,pwd):
	
	_data = [None,[domainName,domainhome,version,adminurl,user,pwd],None]
	util.saveDB(domain=True,data=_data)

def saveServer(name,host,port,type,domain):
	_data = [None,None,[name,host,port,type,domain]]
	util.saveDB(server=True,data=_data)

def saveWLS(wlsHome,version,nmPort):
	_data = [[wlsHome,version,nmPort],None,None]
	util.saveDB(wlss=True,data=_data)


def searchAll():
	print "Searching weblogic and domains on this machine ,please wait ."
	print ""
	if os.path.isfile(util.wlnm_data_file):
		os.remove(util.wlnm_data_file)
	searchwls()
	searchDomain()
	showAll()

def searchwls():
	version='0.0'
	for curdir, dirs, files in os.walk('/'):
	     if  'weblogic.jar' in files and os.path.basename(curdir) == 'lib' and os.path.basename(os.path.dirname(curdir)) == 'server':
		process = subprocess.Popen("unzip -p %s/weblogic.jar META-INF/MANIFEST.MF" % os.path.abspath(curdir),shell=True, stdout=subprocess.PIPE)
		out_str, err = process.communicate()

		for line in out_str.split('\n'):
			#print 'line is ' + line
			if 'Implementation-Version'in line :
				version =  line.split(":")[-1].strip()
		
		wls_home = os.path.abspath(os.path.join(os.path.dirname(curdir), os.pardir))

		nmPort = searchNM(wls_home)
		
		if not nmPort:
			nmPort ='NA'

		saveWLS(wls_home , version , nmPort)

	
def getwlsHome(version):

	wlsDB,wls = util.loadDB(wlss=True)[0]

	for w in wls:
	    if version == wlsDB[w]["version"]:
		return wlsDB[w]["home"]
	return

def getwls(name):
	#logging.debug ("name is %s" % name)
	[wlsDB,wls],[domainsDB,domains],[serversDB,servers] = util.loadDB(wlss=True,domain=True,server=True)
	version = ''
	domain = ''
	wlsHome = ''
	adminurl =''
	user = ''
	pwd = ''
	for server in servers:
		if name == serversDB[server]["name"] :
			domain = serversDB[server]["domain"]
	if domain :
		for idomain in domains:
			if domain == idomain :
				version = domainsDB[idomain]["version"]
				adminurl = domainsDB[idomain]["adminurl"]
				user = domainsDB[idomain]["user"]
				pwd = domainsDB[idomain]["pwd"]
	else:
		#print "could not find such server in local db , make sure you spell correct"
		return ('','','','')

	for key in wls:
		if version == wlsDB[key]["version"]:
			wlsHome = wlsDB[key]["home"]

	return (adminurl,wlsHome,user,pwd)	

def searchDomain():
	for curdir, dirs, files in os.walk('/'):
	     if  'config.xml' in files and os.path.basename(curdir) == 'config' :
		  #print "domain name :%s" % os.path.basename(os.path.dirname(curdir))
		  
		  homedir = os.path.dirname(curdir)
		  
		  
		  tree = ET.parse(curdir+"/config.xml")
		  root = tree.getroot()

		  namespace = get_xml_namespace(root)
		  domainname = root.findall(".//%sname" % namespace)[0].text

		  domainversion = root.findall(".//%sdomain-version" % namespace)[0].text


		  servers = root.findall(".//%sserver" % namespace)
		  #logging.debug (servers)
		  adminserver = root.findall(".//%sadmin-server-name" % namespace)[0].text
		  user = root.findall(".//%ssecurity-configuration/%snode-manager-username" % (namespace,namespace))[0].text
		  #print user
		  enPass= root.findall(".//%ssecurity-configuration/%snode-manager-password-encrypted" % (namespace,namespace))[0].text
		  #print enPass
		  adminPort = ''
		  adminHost = ''
		  for server in servers :
			_name = server.findall(".//%sname" % namespace)[0].text
			_host = server.findall(".//%slisten-address" % namespace)[0].text
			_host = "localhost" if not _host else _host
			_port = server.findall(".//%slisten-port" % namespace)[0].text
			_type = 'A' if _name == adminserver else 'M'
			adminPort = _port if _name == adminserver else adminPort
			adminHost = _host if _name == adminserver else adminHost
			#logging.debug (_name,_host,_port,_type,domainname)
			saveServer(_name,_host,_port,_type,domainname)

		  adminurl = "t3://%s:%s" % (adminHost,adminPort)

		  wlsHome = getwlsHome(domainversion)
		  pwd = wls.getAdminPass(homedir,enPass,wlsHome)
		  saveDomain(domainname,homedir,domainversion,adminurl,user,pwd)


	print ""
	print "Search completed!"
	print ""
	

def show(args = []):
	if not args :
		showAll()
	else:
		print ""
		showServers(args)

def showAll():
	domainsDB,domains = util.loadDB(domain=True)[1]
	print ""
	for domain in domains:
	    showServers([domain])

	

def showDomains():
	
	table = [["Name", "Version", "HomeDirectory","AdminURL"]]
	
	domainsDB,domains = util.loadDB(domain=True)[1]
	for domain in domains:
	    table.append([domain,domainsDB[domain]["version"],domainsDB[domain]["home"],domainsDB[domain]["adminurl"]])
	    
	util.pprint_table(table)


def showServers(domain):
	
	serversDB,servers = util.loadDB(server=True)[2]

	for _domain in domain:
		print "[[%s]]" % _domain
		table = [["Name", "Type", "HOST/IP","PORT","STATUS"]]
		for server in servers:
			if _domain == serversDB[server]["domain"] :
				table.append([serversDB[server]["name"],serversDB[server]["type"],serversDB[server]["host"],serversDB[server]["port"],monitor.isRunning(serversDB[server]["name"])])
		    
		util.pprint_table(table)
		print ""
	
	

def showWLS():
	table = [["Name", "Version", "HomeDirectory" , "NM Port","NM Status"]]
	
	
	wlsDB,wls = util.loadDB(wlss=True)[0]

	_status = 'DOWN'

	for w in wls:
	    if util.checkport(wlsDB[w]["nmport"]):
		_status = 'UP'
	    table.append([w,wlsDB[w]["version"],wlsDB[w]["home"],wlsDB[w]["nmport"], _status])
	    
	util.pprint_table(table)

def getDomain(domainName):
	domainsDB,domains = util.loadDB(domain=True)[1]
	if domainName in domainsDB.keys():
		return domainsDB[domainName]
	else:
		return

def searchNM(wlsHome):
	port = ""
	with open('%s/common/nodemanager/nodemanager.properties' % wlsHome) as f:
		lines = f.read().splitlines()
		for line in lines:
			if "ListenPort" == line.split("=")[0]:
				port = line.split("=")[-1]
	f.close()
	return port			


def checkinit(daemon):
    if not os.path.isfile(util.wlnm_data_file):
	util.creatProfileDir()
        if raw_input("For the first time use weblogic node master, it  need to gather weblogic infos on this machine , proceed [y/n] ?") == 'y' :
		#print 'Search local weblogic domains'
		searchAll()
	else:
		if daemon :
			print "If you run a daemon service must choose Y , quit now."
			print ""
			sys.exit()
		else :
			util.initDB()
			print 'Weblogic node master run as client mode ,use connect command to connect target machine.'
			print ""	

#print searchNM("/bea/1035/wlserver_10.3")
#searchwls()
#showWLS()
#getwls('dsada')


'''

monitor.py
==========

Desc: Function for list all process running on the machine , showing status of each process


Author: Kevin Yi<yikaus @ gmail>

License  : BSD 

'''

import psutil
import datetime

import util

def listprocess():
	#print util.daemonStatus
	

	_table = [["Name", "CPU%", "MEM%" ,"THREADS","UPTIME+","PID"]]
	
	serversDB,servers = util.loadDB(server=True)[2]
	

	for p in psutil.process_iter():
		for subcmd in p.cmdline :
			if '-Dweblogic.Name=' in subcmd :
				_table.append([subcmd.split("=")[-1], "%.2f" % p.get_cpu_percent(), "%.2f" % p.get_memory_percent(), p.get_num_threads(),datetime.datetime.now().replace(microsecond=0)-datetime.datetime.fromtimestamp(p.create_time).replace(microsecond=0),p.pid])
				
				
	util.pprint_table(_table)


def isRunning(server):
	_status = 'DOWN'

	for p in psutil.process_iter():
		if '-Dweblogic.Name=%s' % server in p.cmdline :
			_status = 'UP'

	return _status




#!/usr/bin/env python


from setuptools import setup, find_packages
setup(
    name = "wlnm",
    version = "0.2.0",
    packages = find_packages(),
    

    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
        # And include any *.msg files found in the 'hello' package, too:
        #'hello': ['*.msg'],
	'wlst':['./wlst/*.py'],
    },

    # metadata for upload to PyPI
    author = "Kevin Yi",
    author_email = "yikaus A gmail ",
    description = "Weblogic Node Master",
    license = "BSD",
    keywords = "weblogic admin administrator node manage",
    url = "https://github.com/yikaus/wlnm",   # project home page, if any

    # could also include long_description, download_url, classifiers, etc.

    entry_points = {
        'console_scripts': [
            'wlnm = wlnm.wlnm:main',
        ],
    }
)